package boids;

/**
 * A class representing a flock of Boids.
 * 
 * @author Nir Atias
 *
 */
public class Flock {

	/**
	 * Holds the default parameter values.
	 */
	public static final Flock DEFAULT = new Flock();
	
	private double flockingRate = 0.2;
	private Boid[]	boids;

	// getters
	/**
	 * Get all the boids in the current flock
	 * 
	 * @return An array of boids
	 */
	public Boid[] getBoids() {
		return boids;
	}

	/**
	 * The rate to use when calculating flock related behaviors
	 * (stayWithFlock and maintainFlockVelocity in Boid class).
	 * 
	 * @return The rate. 
	 * 
	 * @see Boid
	 */
	public double getFlockingRate() {
		return flockingRate;
	}

	// setters
	/**
	 * Changes the flocking rate.
	 * @param flockingRate The new rate.
	 * @see Flock#getFlockingRate()
	 */
	public void setFlockingRate(double flockingRate) {
		this.flockingRate = flockingRate;
	}

	// constructors
	/**
	 * Construct a new flock for a group of boids.
	 * 
	 * @param boids The group of boids in the flock. Assume not to be null.
	 */
	/*
	 * @pre boids != null
	 * @pre for all 0 <= i < boids.length : boids[i] != null
	 */
	public Flock(Boid[] boids) {
		this.boids = boids;
		
		for(Boid b : boids)
			b.setFlock(this);
	}

	/**
	 * Constructor used to build the DEFAULT field.
	 */
	private Flock() {
	}

	// operators
	/**
	 * Calculate the position representing the center of mass
	 * for this flock
	 * 
	 * @return The center of mass, defined as the average position.
	 */
	public Point getCenterOfMass() {
		//TODO: Add your code here
	}
	
	/**
	 * Calculates the average velocity of the flock.
	 * 
	 * @return The average velocity.
	 */
	public Point getAvergateVelocity() {
		//TODO: Add your code here
	}
}
