package boids;

/**
 * A class holding a rectangle.
 * 
 * @author Nir Atias
 *
 */
public class Rectangle {
	private double top;
	private double bottom;
	private double left;
	private double right;
	
	/**
	 * Build a new rectangle based on the supplied coordinates.
	 * 
	 * @param top The top coordinate 
	 * @param bottom The bottom coordinate
	 * @param left The left side coordinate
	 * @param right The right side coordinate
	 */
	/*
	 * @pre bottom <= top
	 * @pre left <= right
	 */
	public Rectangle(double top, double bottom, double left, double right) {
		this.top = top;
		this.bottom = bottom;
		this.left = left;
		this.right = right;
	}
	
	// getters
	/**
	 * Returns the top of the rectangle.
	 * @return The top coordinate.
	 */
	public double getTop() {
		return top;
	}
	
	/**
	 * Returns the bottom of the rectangle.
	 * @return The bottom coordinate.
	 */
	public double getBottom() {
		return bottom;
	}
	
	/**
	 * The left of the rectangle.
	 * @return The left coordinate.
	 */
	public double getLeft() {
		return left;
	}
	
	/**
	 * The right side of the rectangle.
	 * @return The right coordinate.
	 */
	public double getRight() {
		return right;
	}
	
	// setters
	/**
	 * Changes the top coordinate.
	 * @param top The new coordinate.
	 */
	public void setTop(double top) {
		this.top = top;
	}

	/**
	 * Changes the bottom coordinate.
	 * @param top The new coordinate.
	 */
	public void setBottom(double bottom) {
		this.bottom = bottom;
	}

	/**
	 * Changes the left side coordinate.
	 * @param top The new coordinate.
	 */
	public void setLeft(double left) {
		this.left = left;
	}

	/**
	 * Changes the right side coordinate.
	 * @param top The new coordinate.
	 */
	public void setRight(double right) {
		this.right = right;
	}
	
	// operations
	
	/**
	 * Tests whether a point is contained inside this rectangle
	 * @param p The requested point
	 * @return True if p is inside this rectangle
	 */
	/*
	 * @pre getRight() < getLeft()
	 * @pre getBottom() < getTop()
	 */
	public boolean isContained(Point p) {
		return p.getX() < right && 
				p.getX() > left && 
				p.getY() < top && 
				p.getY() > bottom;
	}
	
}
