package boids;

import java.util.ArrayList;
import java.util.List;

/**
 * Holds information about the environment in which the Boids
 * move. 
 * 
 * @author Nir Atias
 *
 */
public class Scene {

	private Rectangle limits;

	private List<Boid>	boids = new ArrayList<Boid>();
	
	// getters
	
	/**
	 * The region in which the Boids should move. 
	 * 
	 * @return A Rectangle in which Boids should stay
	 * 
	 * @see Rectangle
	 */
	public Rectangle getLimits() {
		return limits;
	}

	/**
	 * All the boids in the this scene.
	 * 
	 * @return An array of all available Boids.
	 */
	public Boid[] getAllBoids() {
		Boid[] currentBoids = new Boid[boids.size()];
		return boids.toArray(currentBoids);
	}

	// setters
	/**
	 * Change the limits of the current scene.
	 *
	 * Note that the rectangle is used "as-is" and is not copied.
	 * 
	 * @param limits A Rectangle with the appropriate limits
	 */
	public void setLimits(Rectangle limits) {
		this.limits = limits;
	}
	
	/**
	 * Add a Boid to this scene. The "move" method of the Boid will be called
	 * whenever the scene is updated.
	 * 
	 * @param b The boid to add to the scene.
	 */
	public void addBoid(Boid b) {
		boids.add(b);
	}

	/**
	 * Update the scene. This will subsequently instruct all boids to move.
	 */
	public void update() {
		for(Boid b : boids)
			b.move();
	}
	
	/**
	 * Remove all boids from the scene.
	 */
	public void reset() {
		boids = new ArrayList<Boid>();
	}
}
