package boids;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;

public class SimulationParametersPanel extends JPanel {

	private static final long serialVersionUID = 3413206576193370340L;
	private JSpinner boidsSpinner;
	private JSpinner minDistanceSpinner;
	private JSlider returnRateSlider;
	private JSlider flockingRateSlider;
	private ColoredButton colorButton;
	private final JLabel lblMaxVelocity = new JLabel("Max velocity");
	private JSlider velocitySlider;
	private JCheckBox showBackground;
	private JPanel panel;

	static class ColoredButton extends JButton {
		private static final long serialVersionUID = 7198581403571479975L;
		private Color displayColor = null;

		public ColoredButton() {
			super();
		}

		public ColoredButton(Action arg0) {
			super(arg0);
		}

		public ColoredButton(Icon arg0) {
			super(arg0);
		}

		public ColoredButton(String arg0, Icon arg1) {
			super(arg0, arg1);
		}

		public ColoredButton(String arg0) {
			super(arg0);
		}

		@Override
		public void paint(Graphics g) {
			super.paint(g);
			if(displayColor != null) {
				g.setColor(displayColor);
				g.fillRect(5, 5, getWidth()-10, getHeight()-10);
			} 
		}

		public void setDisplayColor(Color background) {
			displayColor  = background;
		}
		
		public Color getDisplayColor() {
			return displayColor;
		}
	}
	/**
	 * Create the panel.
	 */
	public SimulationParametersPanel() {
		setMaximumSize(new Dimension(300, 160));
		setPreferredSize(new Dimension(277, 227));
		setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
		
		panel = new JPanel();
		add(panel);
		GridBagLayout gbl_panel_2 = new GridBagLayout();
		gbl_panel_2.columnWidths = new int[]{78, 154, 0};
		gbl_panel_2.rowHeights = new int[]{20, 0, 0, 0, 0, 0, 0, 0};
		gbl_panel_2.columnWeights = new double[]{0.0, 1.0, Double.MIN_VALUE};
		gbl_panel_2.rowWeights = new double[]{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, Double.MIN_VALUE};
		panel.setLayout(gbl_panel_2);
		
		JLabel lblNumberOfBoids = new JLabel("Number of boids");
		GridBagConstraints gbc_lblNumberOfBoids = new GridBagConstraints();
		gbc_lblNumberOfBoids.anchor = GridBagConstraints.WEST;
		gbc_lblNumberOfBoids.insets = new Insets(0, 0, 5, 5);
		gbc_lblNumberOfBoids.gridx = 0;
		gbc_lblNumberOfBoids.gridy = 0;
		panel.add(lblNumberOfBoids, gbc_lblNumberOfBoids);
		lblNumberOfBoids.setHorizontalAlignment(SwingConstants.LEFT);
		
		boidsSpinner = new JSpinner();
		GridBagConstraints gbc_boidsSpinner = new GridBagConstraints();
		gbc_boidsSpinner.anchor = GridBagConstraints.EAST;
		gbc_boidsSpinner.insets = new Insets(0, 0, 5, 0);
		gbc_boidsSpinner.gridx = 1;
		gbc_boidsSpinner.gridy = 0;
		panel.add(boidsSpinner, gbc_boidsSpinner);
		boidsSpinner.setMaximumSize(new Dimension(29, 20));
		boidsSpinner.setModel(new SpinnerNumberModel(5, 5, 35, 1));
		
		JLabel lblDiameter = new JLabel("Collision distance");
		GridBagConstraints gbc_lblDiameter = new GridBagConstraints();
		gbc_lblDiameter.insets = new Insets(0, 0, 5, 5);
		gbc_lblDiameter.gridx = 0;
		gbc_lblDiameter.gridy = 1;
		panel.add(lblDiameter, gbc_lblDiameter);
		lblDiameter.setHorizontalAlignment(SwingConstants.LEFT);
		
		minDistanceSpinner = new JSpinner();
		GridBagConstraints gbc_minDistanceSpinner = new GridBagConstraints();
		gbc_minDistanceSpinner.anchor = GridBagConstraints.EAST;
		gbc_minDistanceSpinner.insets = new Insets(0, 0, 5, 0);
		gbc_minDistanceSpinner.gridx = 1;
		gbc_minDistanceSpinner.gridy = 1;
		panel.add(minDistanceSpinner, gbc_minDistanceSpinner);
		minDistanceSpinner.setMaximumSize(new Dimension(29, 20));
		minDistanceSpinner.setModel(new SpinnerNumberModel(BoidParameters.DEFAULT.getMinDistance(), 20, 1000, 1));
		
		JLabel lblReturnRate = new JLabel("Return rate");
		GridBagConstraints gbc_lblReturnRate = new GridBagConstraints();
		gbc_lblReturnRate.anchor = GridBagConstraints.WEST;
		gbc_lblReturnRate.insets = new Insets(0, 0, 5, 5);
		gbc_lblReturnRate.gridx = 0;
		gbc_lblReturnRate.gridy = 2;
		panel.add(lblReturnRate, gbc_lblReturnRate);
		
		returnRateSlider = new JSlider();
		GridBagConstraints gbc_returnRateSlider = new GridBagConstraints();
		gbc_returnRateSlider.insets = new Insets(0, 0, 5, 0);
		gbc_returnRateSlider.fill = GridBagConstraints.HORIZONTAL;
		gbc_returnRateSlider.gridx = 1;
		gbc_returnRateSlider.gridy = 2;
		panel.add(returnRateSlider, gbc_returnRateSlider);
		returnRateSlider.setValue((int)Math.round(BoidParameters.DEFAULT.getReturnRate() * 100));
		
		JLabel lblFlockingRate = new JLabel("Flocking rate");
		GridBagConstraints gbc_lblFlockingRate = new GridBagConstraints();
		gbc_lblFlockingRate.anchor = GridBagConstraints.WEST;
		gbc_lblFlockingRate.insets = new Insets(0, 0, 5, 5);
		gbc_lblFlockingRate.gridx = 0;
		gbc_lblFlockingRate.gridy = 3;
		panel.add(lblFlockingRate, gbc_lblFlockingRate);
		
		flockingRateSlider = new JSlider();
		GridBagConstraints gbc_flockingRateSlider = new GridBagConstraints();
		gbc_flockingRateSlider.insets = new Insets(0, 0, 5, 0);
		gbc_flockingRateSlider.fill = GridBagConstraints.HORIZONTAL;
		gbc_flockingRateSlider.gridx = 1;
		gbc_flockingRateSlider.gridy = 3;
		panel.add(flockingRateSlider, gbc_flockingRateSlider);
		flockingRateSlider.setValue((int)Math.round(Flock.DEFAULT.getFlockingRate()*100));
		
		JLabel lblColor = new JLabel("Color");
		GridBagConstraints gbc_lblColor = new GridBagConstraints();
		gbc_lblColor.anchor = GridBagConstraints.WEST;
		gbc_lblColor.insets = new Insets(0, 0, 5, 5);
		gbc_lblColor.gridx = 0;
		gbc_lblColor.gridy = 4;
		panel.add(lblColor, gbc_lblColor);
		
		colorButton = new ColoredButton(" ");
		GridBagConstraints gbc_colorButton = new GridBagConstraints();
		gbc_colorButton.insets = new Insets(0, 0, 5, 0);
		gbc_colorButton.fill = GridBagConstraints.BOTH;
		gbc_colorButton.gridx = 1;
		gbc_colorButton.gridy = 4;
		panel.add(colorButton, gbc_colorButton);
		GridBagConstraints gbc_lblMaxVelocity = new GridBagConstraints();
		gbc_lblMaxVelocity.anchor = GridBagConstraints.WEST;
		gbc_lblMaxVelocity.insets = new Insets(0, 0, 5, 5);
		gbc_lblMaxVelocity.gridx = 0;
		gbc_lblMaxVelocity.gridy = 5;
		panel.add(lblMaxVelocity, gbc_lblMaxVelocity);
		
		velocitySlider = new JSlider();
		GridBagConstraints gbc_velocitySlider = new GridBagConstraints();
		gbc_velocitySlider.insets = new Insets(0, 0, 5, 0);
		gbc_velocitySlider.fill = GridBagConstraints.HORIZONTAL;
		gbc_velocitySlider.gridx = 1;
		gbc_velocitySlider.gridy = 5;
		panel.add(velocitySlider, gbc_velocitySlider);
		velocitySlider.setValue((int)Math.round(BoidParameters.DEFAULT.getMaxVelocity()));
		velocitySlider.setMinimum(3);
		velocitySlider.setMaximum(15);
		
		showBackground = new JCheckBox("Show background");
		GridBagConstraints gbc_showBackground = new GridBagConstraints();
		gbc_showBackground.anchor = GridBagConstraints.WEST;
		gbc_showBackground.gridwidth = 2;
		gbc_showBackground.insets = new Insets(0, 0, 0, 5);
		gbc_showBackground.gridx = 0;
		gbc_showBackground.gridy = 6;
		panel.add(showBackground, gbc_showBackground);
		colorButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent paramActionEvent) {
				Color initialBackground = colorButton.getDisplayColor();
				Color background = JColorChooser.showDialog(null,
						"Choose Color", initialBackground);
				if (background != null) {
					colorButton.setDisplayColor(background);
				}
			}
		});
	}
	
	public void addActionListener(ActionListener l) {
		showBackground.addActionListener(l);
	}

	public int getNumberOfBoids() {
		return (Integer)boidsSpinner.getValue(); 
	}

	public int getMinDistance() {
		return (Integer)minDistanceSpinner.getValue();
	}
	
	public double getReturnRate() {
		return (double)returnRateSlider.getValue() / returnRateSlider.getMaximum();
	}
	
	public double getFlockingRate() {
		return (double)flockingRateSlider.getValue() / flockingRateSlider.getMaximum();
	}
	
	public Color getFlockColor() {
		return colorButton.getDisplayColor();
	}
	
	public int getMaxVelocity() {
		return velocitySlider.getValue();
	}
	
	public boolean isShowBackground() {
		return showBackground.isSelected();
	}
	
	public void setFlockColor(Color color) {
		colorButton.setDisplayColor(color);
		colorButton.repaint();
	}
	
	public void setShowBackground(boolean state) {
		showBackground.setSelected(state);
	}
}
